#include <time.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/stat.h>
#include <stdlib.h>
#ifndef XVRSDK_STATIC
#define XVRSDK_STATIC
#endif // XVRSDK_STATIC
#include "sdk/xvrsdk.h"
#if defined(_WIN32) || defined(WIN32)
#pragma warning(disable: 4996)
#endif

unsigned long get_file_size(const char *path)
{
    struct stat statbuff;
    if (stat(path, &statbuff) < 0)
    {
        return 0;
    }
    else
    {
        return statbuff.st_size;
    }
}


int _read_file(const char* path, char* out_buf, unsigned int out_len)
{
    size_t n = 0;
    FILE* f = fopen(path, "rb");
    if (!f)
    {
        printf("no such file or directory %s\n", path);
        return -1;
    }

    n = fread(out_buf, 1, out_len, f);
    fclose(f);
    return n;
}

void xvr_sdk_test_register_face_by_image(XSHANDLE *ih)
{
    int i;
    int res = 0;
    int count = 0;
    unsigned long file_size;
    char file_name[] = "F:\\debug\\in\\wenweiquan3.jpg";
    char * img_data = (char*)malloc(1024 * 1024);
    struct XVR_SDK_FaceInfoV2 info;
    struct XVR_SDK_FaceInfoV2 result[10];

    memset(img_data, 0, 1024 * 1024);
    file_size = get_file_size(file_name);
    _read_file(file_name, img_data, file_size);
    printf("file_size=%ld\n", file_size);

    for (i = 0; i < 5; ++i)
    {
        sprintf(info.PersonID, "test_person_%d", i);
        sprintf(info.Name, "测试%d", i);
        info.Gender = Male;
        info.Age = 20 + i;
        res = XVR_SDK_FaceRegister(ih, &info, img_data, file_size);
        printf("XVR_SDK_FaceRegister res=%d\n", res);
    }

    free(img_data);

    res = XVR_SDK_FaceList(ih, 0, 50, result, NULL, &count);
    printf("XVR_SDK_FaceList res=%d count=%d\n", res, count);

    char *pids[50];
    for (i = 0; i < count; ++i)
    {
        pids[i] = result[i].PersonID;
    }
    res = XVR_SDK_FaceUnregister(ih, pids, count);
    printf("XVR_SDK_FaceUnregister count=%d res=%d\n", count, res);

}

void xvr_sdk_test_unregister(XSHANDLE *ih)
{
    char *perspn_ids[2] = { 0 };
    char *pid1 = "12345678901234567890123456789011";
    char *pid2 = "12345678901234567890123456789012";

    strncpy(perspn_ids[0], pid1, strlen(pid1));
    strncpy(perspn_ids[1], pid2, strlen(pid2));
    XVR_SDK_FaceUnregister(ih, perspn_ids, 2);
}

void xvr_sdk_test_get_and_set_device_time(XSHANDLE *ih)
{
    struct XVR_SDK_DevCfgTime cfgtime_set = { 0 };
    struct XVR_SDK_DevCfgTime cfgtime_get = { 0 };

    cfgtime_set.timestamp = time(NULL);
    cfgtime_set.timezone = 28800;//8*3600 beijing timezone
    XVR_SDK_SetDeviceConfig(ih, XVR_SDK_DEVCFG_TIME, (char *)&cfgtime_set, sizeof(cfgtime_set));

    XVR_SDK_GetDeviceConfig(ih, XVR_SDK_DEVCFG_TIME, (char *)&cfgtime_get, sizeof(cfgtime_get));
    printf("cfgtime_get.unix_timestamp: %lld\n", cfgtime_get.timestamp);
    printf("cfgtime_get.timezone:%d\n", cfgtime_get.timezone);
}

int main(void)
{
    int res;
    struct XSHANDLE *ih = NULL;

    res = XVR_SDK_Init();
    assert(0 == res);

    res = XVR_SDK_Create(&ih);
    assert(0 == res);

    res = XVR_SDK_Login(ih, "192.168.104.218", 80, "admin", "admin");
    assert(0 == res);

    xvr_sdk_test_get_and_set_device_time(ih);
    for (res = 0; res < 10; ++res) {
        xvr_sdk_test_register_face_by_image(ih);
    }

    //XVR_SDK_Control_Gateway(ih, 1, 0);

    //res = XVR_SDK_Process(ih, 5000);
    //printf("process res=%d\n", res);

    res = XVR_SDK_Cleanup();
    assert(0 == res);

    return 0;
}
