package app;

import java.util.List;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import com.xvr.Sdk;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.LongByReference;

public class AppUsingJNA {

    public static byte[] LoadImage(String filePath) throws Exception {

        File imageFile = new File(filePath);
        if (!imageFile.exists()) {
            throw new FileNotFoundException();
        }

        BufferedImage image = ImageIO.read(imageFile);
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ImageIO.write(image, "jpg", out);
        out.flush();
        return out.toByteArray();
    }

    public static void testMain(Sdk.AICamera sdk) {
        int res;

        res = sdk.XVR_SDK_Init();
        System.out.printf("sdk init res=%d\n", res);

        LongByReference handleRef = new LongByReference();
        res = sdk.XVR_SDK_Create(handleRef);
        long handle = handleRef.getValue();
        System.out.printf("created handle=%d res=%d\n", handle, res);

        res = sdk.XVR_SDK_Login(handle, "192.168.104.218", 80, "admin", "admin");
        System.out.printf("login res=%d\n", res);

        try {
            Date date = new Date();
            DateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
            String imageFile = AppUsingJNA.class.getResource("testImage.jpg").getPath();
            byte[] faceImage = LoadImage(imageFile);
            Sdk.FaceInfo registerFace = new Sdk.FaceInfo();
            registerFace.setPersonId(sdf.format(date) + "_测试");
            registerFace.setName("测试名字");
            res = sdk.XVR_SDK_FaceRegister(handle, registerFace, faceImage, faceImage.length);
            System.out.printf("register face res=%d\n", res);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        // query first 10 faces
        Sdk.FaceInfo[] queryResult = (Sdk.FaceInfo[]) new Sdk.FaceInfo().toArray(10);
        IntByReference countRef = new IntByReference();
        res = sdk.XVR_SDK_FaceList(handle, 0, 10, queryResult, "", countRef);
        System.out.printf("list face res=%d count=%d\n", res, countRef.getValue());
        for (int i = 0; i < countRef.getValue(); ++i) {
            System.out.printf(" face[%d] PersonId=[%s] Name=[%s]\n", i, queryResult[i].getPersonId(),
                    queryResult[i].getName());
        }

        // delete first 10 registered faces
        List<String> ids = new ArrayList<String>();
        for (int i = 0; i < countRef.getValue(); ++i) {
            ids.add(queryResult[i].getPersonId());
        }
        String[] personIds = ids.toArray(new String[ids.size()]);
        res = sdk.XVR_SDK_FaceUnregister(handle, personIds, ids.size());
        System.out.printf("unregister %d faces res=%d\n", ids.size(), res);

        res = sdk.XVR_SDK_Destroy(handle);
        System.out.printf("destroy handle res=%d\n", res);

        res = sdk.XVR_SDK_Cleanup();
        System.out.printf("sdk cleanup res=%d\n", res);
    }

    public static void main(String[] args) {

        Sdk.AICamera sdk;
        System.out.println("Current OS: " + System.getProperty("os.name"));
        System.out.println("Current ClassPath: " + System.getProperty("java.class.path"));

        File directory = new File(".");
        try {
            System.out.println("Current directory's canonical path: " + directory.getCanonicalPath());
            System.out.println("Current directory's absolute  path: " + directory.getAbsolutePath());
        } catch (Exception e) {
            System.out.println("Exceptione is =" + e.getMessage());
        }

        File classfile = new File(AppUsingJNA.class.getProtectionDomain().getCodeSource().getLocation().getPath());
        System.out.println("Current ClassFilePath: " + classfile.getAbsolutePath());

        if (System.getProperty("os.name").contains("Linux")) {
            sdk = Sdk.GetInterface("libxvrsdkd.so");
        } else {
            sdk = Sdk.GetInterface("xvrsdk-dll-mt-vc140.dll");
        }

        for (int i = 0; i < 1000; ++i) {
            testMain(sdk);
        }
    }

}
