package com.xvr;

import java.util.List;
import java.util.Arrays;

import com.sun.jna.Native;
import com.sun.jna.Library;
import com.sun.jna.Structure;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.LongByReference;

public class Sdk {
    private static int strlen(byte[] cstring) {
        int i;
        for (i = 0; i < cstring.length && cstring[i] != 0; ++i) {
        }
        return i;
    }

    private static void strncpy(byte[] dst, byte[] src, int max) {
        int i;
        for (i = 0; i < src.length && i < max && src[i] != 0; ++i) {
            dst[i] = src[i];
        }
        dst[i + 1] = 0;
    }

    /// 注册人脸信息
    public static class FaceInfo extends Structure {
        public static class ByReference extends FaceInfo implements Structure.ByReference {
        }

        public static class ByValue extends FaceInfo implements Structure.ByValue {
        }

        public byte[] PersonID = new byte[34];
        public byte[] Name = new byte[34];
        public byte[] Label = new byte[65];
        public int Age;
        public int Gender;

        public void setPersonId(String v) {
            strncpy(PersonID, v.getBytes(), 33);
        }

        public void setName(String v) {
            strncpy(Name, v.getBytes(), 33);
        }

        public void setLabel(String v) {
            strncpy(Label, v.getBytes(), 64);
        }

        public String getPersonId() {
            return new String(PersonID, 0, strlen(PersonID));
        }

        public String getName() {
            return new String(Name, 0, strlen(Name));
        }

        public String getLabel() {
            return new String(Label, 0, strlen(Label));
        }

        @Override
        protected List<String> getFieldOrder() {
            return Arrays.asList(new String[] { "PersonID", "Name", "Label", "Age", "Gender" });
        }
    }

    public interface AICamera extends Library {

        /// 初始化SDK
        int XVR_SDK_Init();

        /// 清理SDK
        int XVR_SDK_Cleanup();

        /**
         * @brief 关闭设备连接
         * @param handle 返回设备连接句柄
         * @return 错误码
         */
        int XVR_SDK_Create(LongByReference handle);

        /**
         * @brief 关闭设备连接
         * @param handle 设备连接句柄
         * @return 错误码
         */
        int XVR_SDK_Destroy(Long handle);

        /**
         * @brief 登入设备
         * @param handle 设备连接句柄
         * @param address 设备地址
         * @param port 设备HTTP端口
         * @param username 用户名
         * @param password 密码
         * @return 错误码
         */
        int XVR_SDK_Login(Long handle, String address, int port, String username, String password);

        /**
         * @brief 注册人脸
         * @param handle 设备连接句柄
         * @param info 人脸信息
         * @param imgData 人脸图片
         * @param imgDataLength 人脸图片大小
         * @return 错误码
         */
        int XVR_SDK_FaceRegister(Long handle, FaceInfo info, byte[] imgData, int imgDataLength);

        /**
         * @brief 查询人脸列表
         * @param handle 设备连接句柄
         * @param start 查询记录起始位置
         * @param limit 查询记录上限条数
         * @param result 查询结果
         * @param condition 查询条件, 如非空字符串, 则搜索包含condition的人名或ID
         * @param count 查询结果返回记录数
         * @return 错误码
         */
        int XVR_SDK_FaceList(Long handle, int start, int limit, FaceInfo[] result, String condition,
                IntByReference count);

        /**
         * @brief 反注册人脸
         * @param handle 设备连接句柄
         * @param personId 注册人ID数组
         * @param personIdCount ID数组大小
         * @return 错误码
         */
        int XVR_SDK_FaceUnregister(Long handle, String[] personId, int personIdCount);
    }

    /**
     * 获取SDK接口
     * @param libPath SDK动态库文件路径
     * @return SDK接口
     */
    public static AICamera GetInterface(String libPath) {
        return (AICamera) Native.loadLibrary(libPath, AICamera.class);
    }

}
